#!BPY
# -*- coding: latin-1 -*-

# -------------------------------------------------------------------------- 
#  File: alc_PreParse.py 
#  Date: 2011_04_18
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# Copyright (C) 2011: Rainer Kesselschlaeger aka Tachzusamm
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 


import sys, os

try:
    import Blender
except ImportError:
    pass

# For the normal parsing process, yaml ist used; unfortunately, the yaml parser does not store any comment lines
# as well, so they would be lost when re-writing the AlcScript to Blender. Additionally, the order of objects gets
# totally shuffled.
# To fix these issues, I created this PreParser module.
# It reads the complete AlcScript before it gets parsed, stores comment lines and the initial order of the objects.
# The top level object names are just stored in a list (not a dict), together with the leading comment lines.
# On a re-write, this list is used to find the original object order and to insert the belonging comments again.

def aw_PreParse_AlcScript():
    objectList = []
    comment = ""	# comment lines leading the object
    obName = None	
    obComm = ""		# comment lines inside the object
    obText = ""		# the object itself, in AlcScript
    workOb = None
    
    #blendtext = alcFindBlenderText(AlcScript.ObjectTextFileName)
    try:
        AlcText = Blender.Text.Get("AlcScript")
    except NameError:
        AlcText = Blender.Text.New("AlcScript")
    
    for line in AlcText.asLines():
        #print line
        if isCommentLine(line):
            comment += line + "\n"
            if obName is not None:
                obComm += line + "\n"
            
        elif len(line.strip()) == 0:	# treat completely empty lines as comment as well
            comment += line + "\n"
            if obName is not None:
                obComm += line + "\n"
        
        elif isObjectNameLine(line):
            obName = commStrip(line).rstrip(':')
            
            # close previous object and append it (if exists)
            if workOb is not None:
                objectList.append((workOb[0], workOb[1], obText))
            
            # begin collection of next (this) object
            workOb = (obName, comment)
            comment = ""
            obComm = ""
            obText = line + "\n"
            
        else:	# data belonging to the actual object structure
            comment = ""
            if len(obComm) > 0:
                obText += obComm
                obComm = ""
            obText += line + "\n"
        
        	
    # when lines are done, append the last object
    if workOb is not None:
        objectList.append((workOb[0], workOb[1], obText))
    
    # finally append the last comment to an empty object
    if len(comment) > 0:
        objectList.append(("", comment, ""))
    
    
    # Note: After his has run, the objectList should contain the complete AlcScript
    #       in this form: obList[0][1] (comment) + obList[0][2] (object)
    #                   + obList[1][1] (comment) + obList[1][2] (object)
    #                   + ... (up to obList[n])
    #print objectList
    
    # Print a test output (DEBUGGING)
    if False:	# (TODO: set to False in final release to avoid unwanted file creation)
        textfile = open("C:\\PRE_PARSE_DEBUG.txt", "w")
        for obTupel in objectList:
            textfile.write("%s" % obTupel[1])
            textfile.write("%s" % obTupel[2])
        textfile.close()
        # (this should reflect the original AlcScript)
    
    return objectList
    


# Helper functions
# ================

# Checks if we have a comment line; criteria: must start with a '#'
# -----------------------------------------------------------------
def isCommentLine(line):
    txt = line.strip()
    return txt.startswith('#')
    

# Checks if a line is an object name (top level); criteria: starts in first column, ends with ':'
# -----------------------------------------------------------------------------------------------
def isObjectNameLine(line):
    # first remove the trailing comment - if any
    txt = commStrip(line)
    #print txt
    
    # do the comparison
    if txt.endswith(':'):
        txt = txt.lstrip()
        if line[:len(txt)] == txt:
            return True
    return False
    

# Removes a comment on the right side and trims it same side
# ----------------------------------------------------------
def commStrip(line):
    comPos = line.find('#')
    if comPos >= 0:
        txt = line[:comPos].rstrip()
        return txt
    return line.rstrip()
    
